/***************************************************************************                                              
 *   Copyright (C) 2011 Joven Club de Computacion y Electronica V.C        *                                      
 *   Written by Darien Alonso Camacho <darienad030111@vcl.jovenclub.cu>    *                                         
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
 **************************************************************************/

#include "frame.h"
#include "timelinelayer.h"

struct Point
{
    double x;
    double y;
};

Frame::Frame( TimelineLayer* parent_layer, SWFMovie* movie )
{
    this->duration = 1;
    this->parent_layer = parent_layer;
    this->movie = movie;
}

void Frame::setDuration( int duration )
{
    this->duration = duration;
}

int Frame::getDuration()
{
    return this->duration;
}

void Frame::addElement( TiXmlElement* element )
{
    if( element )
    {
        items_to_draw.push_back( element );
    }
}

//Returns the decimal representation of the hexadecimal char c
int Frame::hexToDec( char c )
{
    int converted = -1;
    switch( c )
    {
        case '0': converted = 0; break;
        case '1': converted = 1; break;
        case '2': converted = 2; break;
        case '3': converted = 3; break;
        case '4': converted = 4; break;
        case '5': converted = 5; break;
        case '6': converted = 6; break;
        case '7': converted = 7; break;
        case '8': converted = 8; break;
        case '9': converted = 9; break;
        //Letters A-F are always in upper case in the cz format
        case 'A': converted = 10; break;
        case 'B': converted = 11; break;
        case 'C': converted = 12; break;
        case 'D': converted = 13; break;
        case 'E': converted = 14; break;
        case 'F': converted = 15; break;
    }

    return converted;
}
//---------------------------------------------------------------------------------------
/*
  Decouples the color rgb_str in its red, green and blue components.
  rgb_str is in color string format "#RRGGBB".
  When function ends r, g, b arguments holds rgb components in its decimal representation
*/
void Frame::rgbStrToRgbDec( const char* rgb_str, int& r, int& g, int& b )
{
    char rgb_array[ 8 ];
    strcpy( rgb_array, rgb_str );

    int r1 = Frame::hexToDec( rgb_array[ 1 ] );
    int r2 = Frame::hexToDec( rgb_array[ 2 ] );
    int g1 = Frame::hexToDec( rgb_array[ 3 ] );
    int g2 = Frame::hexToDec( rgb_array[ 4 ] );
    int b1 = Frame::hexToDec( rgb_array[ 5 ] );
    int b2 = Frame::hexToDec( rgb_array[ 6 ] );

    r = 16 * r1 + r2;
    g = 16 * g1 + g2;
    b = 16 * b1 + b2;
}
//------------------------------------------------------------------------------------------------
//Draws a circle in the swf movie, circle properties are specified by the "element" param
void Frame::drawCircle( TiXmlElement* element )
{
    int p_width, p_red, p_green, p_blue = 0;    //pen style
    int f_red, f_green, f_blue = 0;             //fill style
    int depth = 0;				//the elevation of the item
    double x, y, r = 0.0;
    TiXmlAttribute* attrib = 0;

    attrib = element->FirstAttribute();
    if( !attrib )
    {
        cout<< "cz2swf: Circle without attributes, ignored." << endl;
        return;
    }

    while( attrib )
    {
        if( strcmp( attrib->Name(), "penColor" ) == 0 )
        {
            const char* rgb_color_str = attrib->Value();
            rgbStrToRgbDec( rgb_color_str, p_red, p_green, p_blue );
        }
        if( strcmp( attrib->Name(), "penWidth" ) == 0 )
        {
            p_width = attrib->IntValue();
        }
        if( strcmp( attrib->Name(), "fillColor" ) == 0 )
        {
            const char* rgb_color_str = attrib->Value();
            rgbStrToRgbDec( rgb_color_str, f_red, f_green, f_blue );
        }
        if( strcmp( attrib->Name(), "x" ) == 0 )
        {
            x = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "y" ) == 0 )
        {
            y = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "r" ) == 0 )
        {
            r = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "depth" ) == 0 )
        {
            depth = attrib->IntValue();
        }
        attrib = attrib->Next();
    }//End while

    //Create circle shape
    //alpha component 255 by default
    SWFFillStyle* solidfillstyle = SWFFillStyle::SolidFillStyle( f_red, f_green, f_blue );
    SWFShape* circle = new SWFShape();
    circle->setRightFillStyle( solidfillstyle );
    circle->setLineStyle( p_width, p_red, p_green, p_blue );    //alpha by default
    circle->drawCircle( r );

    SWFDisplayItem* circleDisplayItem = movie->add( circle );
    circleDisplayItem->moveTo( x, y );
    circleDisplayItem->setDepth( depth );

    //
    //if( this->duration > 0 )
        parent_layer->addItemToBeRemoved( this->duration, circleDisplayItem );
}
//---------------------------------------------------------------------------------------------
//Draws an ellipse in the swf movie, ellipse properties are specified by the "element" param
void Frame::drawEllipse( TiXmlElement* element )
{
    int p_width, p_red, p_green, p_blue = 0;    //pen style
    int f_red, f_green, f_blue = 0;             //fill style
    int depth = 0;				//the elevation of the item
    double x_center, y_center = 0.0;
    double bound_rect_width = 0.0;
    double bound_rect_height = 0.0;
    TiXmlAttribute* attrib = 0;

    attrib = element->FirstAttribute();
    if( !attrib )
    {
        cout<< "cz2swf: Ellipse without attributes, ignored." << endl;
        return;
    }

    while( attrib )
    {
        if( strcmp( attrib->Name(), "penColor" ) == 0 )
        {
            const char* rgb_color_str = attrib->Value();
            rgbStrToRgbDec( rgb_color_str, p_red, p_green, p_blue );
        }
        if( strcmp( attrib->Name(), "penWidth" ) == 0 )
        {
            p_width = attrib->IntValue();
        }
        if( strcmp( attrib->Name(), "fillColor" ) == 0 )
        {
            const char* rgb_color_str = attrib->Value();
            rgbStrToRgbDec( rgb_color_str, f_red, f_green, f_blue );
        }
        if( strcmp( attrib->Name(), "x" ) == 0 )
        {
            x_center = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "y" ) == 0 )
        {
            y_center = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "width" ) == 0 )
        {
            bound_rect_width = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "height" ) == 0 )
        {
            bound_rect_height = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "depth" ) == 0 )
        {
            depth = attrib->IntValue();
        }
        attrib = attrib->Next();
    }

    SWFFillStyle* solidFillStyle = SWFFillStyle::SolidFillStyle( f_red, f_green, f_blue );

    SWFShape* ellipse = new SWFShape();
    ellipse->setRightFillStyle( solidFillStyle );
    ellipse->setLineStyle( p_width, p_red, p_green, p_blue );

    double x = 0.0;
    double y = 0.0;
    double width = bound_rect_width / 2;
    double height = bound_rect_height / 2;

    double j =  width * 0.70711;
    double n = height * 0.70711;
    double i = j - (height - n) * width / height;
    double m = n - (width - j) * height / width;

    ellipse->movePenTo( x + width, y );
    ellipse->drawCurveTo( x + width, y - m, x + j, y - n );
    ellipse->drawCurveTo( x + i, y - height, x, y - height );
    ellipse->drawCurveTo( x - i, y - height, x - j, y - n );
    ellipse->drawCurveTo( x - width, y - m, x - width, y );
    ellipse->drawCurveTo( x - width, y + m, x - j, y + n );
    ellipse->drawCurveTo( x - i, y + height, x, y + height );
    ellipse->drawCurveTo( x + i, y + height, x + j, y + n );
    ellipse->drawCurveTo( x + width, y + m, x + width, y );

    SWFDisplayItem* ellipseDisplayItem = movie->add( ellipse );
    ellipseDisplayItem->moveTo( x_center, y_center );
    ellipseDisplayItem->setDepth( depth );

    //
    //if( this->duration > 0 )
        parent_layer->addItemToBeRemoved( this->duration, ellipseDisplayItem );
}
//--------------------------------------------------------------------------------------------
//Draws a rectangle in the swf movie, rectangle properties are specified by the "element" param
void Frame::drawRect( TiXmlElement* element )
{
    int p_width, p_red, p_green, p_blue = 0;    //pen style
    int f_red, f_green, f_blue = 0;             //fill style
    int depth = 0;				//the elevation of the item
    double x1, y1, x2, y2 = 0;
    double bound_rect_width = 0.0;
    double bound_rect_height = 0.0;
    TiXmlAttribute* attrib = 0;

    attrib = element->FirstAttribute();
    if( !attrib )
    {
        cout<< "cz2swf: Rectangle without attributes, ignored." << endl;
        return;
    }

    while( attrib )
    {
        if( strcmp( attrib->Name(), "penColor" ) == 0 )
        {
            const char* rgb_color_str = attrib->Value();
            rgbStrToRgbDec( rgb_color_str, p_red, p_green, p_blue );
        }
        else if( strcmp( attrib->Name(), "penWidth" ) == 0 )
        {
            p_width = attrib->IntValue();
        }
        else if( strcmp( attrib->Name(), "fillColor" ) == 0 )
        {
            const char* rgb_color_str = attrib->Value();
            rgbStrToRgbDec( rgb_color_str, f_red, f_green, f_blue );
        }
        else if( strcmp( attrib->Name(), "x1" ) == 0 )
        {
            x1 = attrib->DoubleValue();
        }
        else if( strcmp( attrib->Name(), "y1" ) == 0 )
        {
            y1 = attrib->DoubleValue();
        }
        else if( strcmp( attrib->Name(), "x2" ) == 0 )
        {
            x2 = attrib->DoubleValue();
        }
        else if( strcmp( attrib->Name(), "y2" ) == 0 )
        {
            y2 = attrib->DoubleValue();
        }
        else if( strcmp( attrib->Name(), "width" ) == 0 )
        {
            bound_rect_width = attrib->DoubleValue();
        }
        else if( strcmp( attrib->Name(), "height" ) == 0 )
        {
            bound_rect_height = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "depth" ) == 0 )
        {
            depth = attrib->IntValue();
        }
        attrib = attrib->Next();
    }
    SWFFillStyle* solidfillstyle = SWFFillStyle::SolidFillStyle( f_red, f_green, f_blue );

    SWFShape* rectShape = new SWFShape();
    rectShape->setLineStyle( p_width, p_red, p_green, p_blue );
    rectShape->setRightFillStyle( solidfillstyle );

    rectShape->movePenTo( x1, y1 );
    rectShape->drawLineTo( x1, y2 );
    rectShape->drawLineTo( x2, y2 );
    rectShape->drawLineTo( x2, y1 );
    rectShape->drawLineTo( x1, y1 );

    SWFDisplayItem* rectDisplayItem = movie->add( rectShape );
    rectDisplayItem->setDepth( depth );

    //
    //if( this->duration > 0 )
        parent_layer->addItemToBeRemoved( this->duration, rectDisplayItem );
}
//--------------------------------------------------------------------------------------------
//Draws a path in the swf movie, path properties are specified by the "element" param
void Frame::drawPath( TiXmlElement* element )
{
    int p_width, p_red, p_green, p_blue = 0;    //pen style
    int f_red, f_green, f_blue = 0;             //fill style
    int depth = 0;				//the elevation of the item
    double x, y = 0.0;
    TiXmlAttribute* attrib = 0;
    TiXmlElement* child_element = 0;
    vector<Point> points;

    attrib = element->FirstAttribute();
    if( !attrib )
    {
        cout<< "cz2swf: Path without attributes, ignored." << endl;
        return;
    }

    while( attrib )
    {
        if( strcmp( attrib->Name(), "penColor" ) == 0 )
        {
            const char* rgb_color_str = attrib->Value();
            rgbStrToRgbDec( rgb_color_str, p_red, p_green, p_blue );
        }
        else if( strcmp( attrib->Name(), "penWidth" ) == 0 )
        {
            p_width = attrib->IntValue();
        }
        else if( strcmp( attrib->Name(), "fillColor" ) == 0 )
        {
            const char* rgb_color_str = attrib->Value();
            rgbStrToRgbDec( rgb_color_str, f_red, f_green, f_blue );
        }
        if( strcmp( attrib->Name(), "depth" ) == 0 )
        {
            depth = attrib->IntValue();
        }
        attrib = attrib->Next();
    }

    child_element = element->FirstChildElement();
    if( !child_element )
    {
        cout<< "cz2swf: Path without points, ignored." << endl;
        return;
    }

    while( child_element )
    {
        if( child_element->ValueStr() == "point" )
        {
            attrib = child_element->FirstAttribute();
            if( !attrib )
            {
                cout<< "cz2swf: Point without attributes, ignored." << endl;
                return;
            }

            while( attrib )
            {
                if( strcmp( attrib->Name(), "x" ) == 0 )
                {
                    x = attrib->DoubleValue();
                }
                else if( strcmp( attrib->Name(), "y" ) == 0 )
                {
                    y = attrib->DoubleValue();
                }
                attrib = attrib->Next();
            }
            Point p;
            p.x = x;
            p.y = y;
            points.push_back( p );
        }
        else
        {
            cout<< "cz2swf: Bad document structure, no <point> tag." << endl;
        }
        child_element = child_element->NextSiblingElement();
    }

    SWFFillStyle* solidfillstyle = SWFFillStyle::SolidFillStyle( f_red, f_green, f_blue );

    SWFShape* pathShape = new SWFShape();
    pathShape->setLineStyle( p_width, p_red, p_green, p_blue );
    pathShape->setRightFillStyle( solidfillstyle );

    Point p = points.at( 0 );
    x = p.x;
    y = p.y;

    pathShape->movePenTo( x, y );

    p = points.at( 6 );
    pathShape->drawLineTo( p.x, p.y );
    p = points.at( 1 );
    pathShape->drawLineTo( p.x, p.y );
    p = points.at( 5 );
    pathShape->drawLineTo( p.x, p.y );
    p = points.at( 3 );
    pathShape->drawLineTo( p.x, p.y );
    p = points.at( 7 );
    pathShape->drawLineTo( p.x, p.y );
    p = points.at( 2 );
    pathShape->drawLineTo( p.x, p.y );
    p = points.at( 4 );
    pathShape->drawLineTo( p.x, p.y );

    pathShape->drawLineTo( x, y );

    SWFDisplayItem* pathDisplayItem = movie->add( pathShape );
    pathDisplayItem->setDepth( depth );

    //
    //if( this->duration > 0 )
        parent_layer->addItemToBeRemoved( this->duration, pathDisplayItem );
}
//--------------------------------------------------------------------------------------------
//Draws a line in the swf movie, line properties are specified by the "element" param
void Frame::drawLine( TiXmlElement* element )
{
    int p_width, p_red, p_green, p_blue = 0;    //pen style
    int depth = 0;				//the elevation of the item
    double x1, y1, x2, y2 = 0.0;
    TiXmlAttribute* attrib = 0;

    attrib = element->FirstAttribute();
    if( !attrib )
    {
        cout<< "cz2swf: Line without attributes, ignored." << endl;
        return;
    }

    while( attrib )
    {
        if( strcmp( attrib->Name(), "penColor" ) == 0 )
        {
            const char* rgb_color_str = attrib->Value();
            rgbStrToRgbDec( rgb_color_str, p_red, p_green, p_blue );
        }
        if( strcmp( attrib->Name(), "penWidth" ) == 0 )
        {
            p_width = attrib->IntValue();
        }
        if( strcmp( attrib->Name(), "x1" ) == 0 )
        {
            x1 = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "y1" ) == 0 )
        {
            y1 = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "x2" ) == 0 )
        {
            x2 = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "y2" ) == 0 )
        {
            y2 = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "depth" ) == 0 )
        {
            depth = attrib->IntValue();
        }

        attrib = attrib->Next();
    }//End while

    //Create line shape
    SWFShape* lineShape = new SWFShape();
    lineShape->setLineStyle( p_width, p_red, p_green, p_blue );    //alpha by default

    lineShape->movePenTo( x1, y1 );
    lineShape->drawLineTo( x2, y2 );
    SWFDisplayItem* lineDisplayItem = movie->add( lineShape );
    lineDisplayItem->setDepth( depth );

    //
    //if( this->duration > 0 )
        parent_layer->addItemToBeRemoved( this->duration, lineDisplayItem );
}
//--------------------------------------------------------------------------------------------
//Draws an image in the swf movie, image properties are specified by the "element" param
void Frame::drawImage( TiXmlElement* element )
{
    const char* path = 0;
    double x, y = 0.0;
    int depth = 0;				//the elevation of the item
    TiXmlAttribute* attrib = 0;

    attrib = element->FirstAttribute();
    if( !attrib )
    {
        cout<< "cz2swf: Image without attributes, ignored." << endl;
        return;
    }

    while( attrib )
    {
        if( strcmp( attrib->Name(), "x" ) == 0 )
        {
            x = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "y" ) == 0 )
        {
            y = attrib->DoubleValue();
        }
        if( strcmp( attrib->Name(), "path" ) == 0 )
        {
            path = attrib->Value();
        }
        if( strcmp( attrib->Name(), "depth" ) == 0 )
        {
            depth = attrib->IntValue();
        }

        attrib = attrib->Next();
    }//End while

    //Create image shape
    SWFBitmap* image;
    try
    {
        image = new SWFBitmap( path );
    }
    catch( SWFException e )
    {
        cout<< e.what() << endl;
    }

    SWFDisplayItem* imageDisplayItem = movie->add( image );
    imageDisplayItem->moveTo( x, y );
    imageDisplayItem->setDepth( depth );

    //
    //if( this->duration > 0 )
        parent_layer->addItemToBeRemoved( this->duration, imageDisplayItem );

}

void Frame::addItemToBeRemoved( SWFDisplayItem* item )
{
    if( item )
    {
        items_to_remove.push_back( item );
    }
}

void Frame::removeItemsToBeRemoved()
{
    for( unsigned int j = 0; j < items_to_remove.size(); j ++ )
    {cout<< "paso" << endl;
        this->movie->remove( items_to_remove.at( j ) );
    }

    items_to_remove.clear();
}

void Frame::draw()
{
    for( int i = 0; i < items_to_draw.size(); i ++ )
    {
        TiXmlElement* element = items_to_draw.at( i );

        if( element->ValueStr() == "script" )
        {

        }
        else if( element->ValueStr() == "symbolInstance" )
        {

        }
        else if( element->ValueStr() == "ellipse" )
        {
            this->drawEllipse( element );
        }
        else if( element->ValueStr() == "circle" )
        {
            this->drawCircle( element );
        }
        else if( element->ValueStr() == "rect" )
        {
            this->drawRect( element );
        }
        else if( element->ValueStr() == "image" )
        {
            this->drawImage( element );
        }
        else if( element->ValueStr() == "path" )
        {
            this->drawPath( element );
        }
        else if( element->ValueStr() == "line" )
        {
            this->drawLine( element );
        }
    }
}














