/***************************************************************************                                              
 *   Copyright (C) 2011 Joven Club de Computacion y Electronica V.C        *                                      
 *   Written by Darien Alonso Camacho <darienad030111@vcl.jovenclub.cu>    *                                         
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
 **************************************************************************/

#ifndef FRAME_H
#define FRAME_H

#include <vector>
#include <mingpp.h>
#include "../TinyXml/tinyxml.h"
using namespace std;

class SWFCharacter;
class SWFDisplayItem;
class TimelineLayer;

class Frame
{
public:
    Frame( TimelineLayer* parent_layer, SWFMovie* movie );
    void addElement( TiXmlElement* element );
    void addItemToBeRemoved( SWFDisplayItem* item );
    void removeItemsToBeRemoved();
    void setDuration( int duration );
    int getDuration();
    void draw();

    //Returns the decimal representation of the hexadecimal char c
    static int hexToDec( char c );

    /*
      Decouples the color rgb_str in its red, green and blue components.
      rgb_str is in color string format "#RRGGBB".
      When function ends r, g, b arguments holds rgb components in its decimal representation
    */
    static void rgbStrToRgbDec( const char* rgb_str, int& r, int& g, int& b );

private:
    //Draws a circle in the swf movie, circle properties are specified by the "element" param
    void drawCircle( TiXmlElement* element );

    //Draws an ellipse in the swf movie, ellipse properties are specified by the "element" param
    void drawEllipse( TiXmlElement* element );

    //Draws a rectangle in the swf movie, rectangle properties are specified by the "element" param
    void drawRect( TiXmlElement* element );

    //Draws a path in the swf movie, path properties are specified by the "element" param
    void drawPath( TiXmlElement* element );

    //Draws a line in the swf movie, line properties are specified by the "element" param
    void drawLine( TiXmlElement* element );

    //Draws an image in the swf movie, image properties are specified by the "element" param
    void drawImage( TiXmlElement* element );

    //Items to draw in the swf movie at this frame
    vector<TiXmlElement*> items_to_draw;
    //Items to remove in the swf movie at this frame
    vector<SWFDisplayItem*> items_to_remove;

    int duration;
    TimelineLayer* parent_layer;
    SWFMovie* movie;
};

#endif // FRAME_H
