/***************************************************************************                                              
 *   Copyright (C) 2011 Joven Club de Computacion y Electronica V.C        *                                      
 *   Written by Darien Alonso Camacho <darienad030111@vcl.jovenclub.cu>    *                                         
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
 **************************************************************************/

#include "timeline.h"

Timeline::Timeline( SWFMovie* movie  )
{
    this->movie = movie;
}

void Timeline::addLayer()
{
    TimelineLayer* l = new TimelineLayer( this->movie );
    layers.push_back( l );
}

void Timeline::completeFrames( int layer_pos , int frame_pos, int duration )
{
    TimelineLayer* layer = layers.at( layer_pos );
    if( layer )
    {
        layer->completeFrames( frame_pos, duration );
    }
}

void Timeline::addElement( int layer_pos , int frame_pos, TiXmlElement* element )
{
    TimelineLayer* layer = layers.at( layer_pos );
    if( layer )
    {
        layer->addElement( frame_pos, element );
    }
}

void Timeline::setFrameDuration( int layer_pos , int frame_pos, int duration )
{
    TimelineLayer* layer = layers.at( layer_pos );
    if( layer )
    {
        layer->setFrameDuration( frame_pos, duration );
    }
}

void Timeline::dumpMovie()
{
    TimelineLayer* current_layer = 0;
    Frame* current_frame = 0;
    int max_frames = 0;
    int layers_count = layers.size();

    //
    for( int i = 0; i < layers_count; i ++ )
    {
        current_layer = layers.at( i );
        if( current_layer->getFramesCount() > max_frames )
        {
            max_frames = current_layer->getFramesCount();
        }
    }
cout<< "max_frames= " << max_frames << endl;
    //Iterate over all frames...
    for ( int k = 0; k < max_frames - 1; k ++ )
    {
        //...by column
        for( int f = 0; f < layers_count; f ++ )
        {
            current_layer = layers.at( f );
            if( k < current_layer->getFramesCount() )
            {
                current_frame = current_layer->getFrame( k );
                if( current_frame )
                {//cout<< k << endl;
                    current_frame->removeItemsToBeRemoved();
                    current_frame->draw();
                }
            }
        }

        this->movie->nextFrame();
    }
}
